<?php

    class postaccesscontroller_checks{

    	public function __construct($pac_db, $pac_ui){
    		$this->db = $pac_db;
			add_action( 'the_post'     , array( $this, 'post_check' ), 10, 1 );
			add_filter( 'the_content'  , array( $this, 'content_check' ), 10, 1 );
			add_filter( 'posts_join'   , array( $this, 'posts_join') );
			add_filter( 'posts_where'  , array( $this, 'posts_where') );
    	}

    	private function no_access_msg( $post_id ){
	        $msg_type = get_post_meta( $post_id, 'postaccesscontroller_noacs_msg_type', true);

	        //change the post object to show the message the user setup
	        if( $msg_type == 'std' ):
	        	return get_option('access_denied_message');
	        else:
	            return get_post_meta( $post_id, 'postaccesscontroller_noacs_custom_msg', true);
	        endif;
    	}

    	public function content_check( $content ){

    		global $post;

    		if( !$this->db->post_access_allow_check( $post ) ){
    			return $this->no_access_msg( $post->ID );
    		}

    		return $content;

    	}

		public function post_check( $post_obj ){

			// echo '13';

		    if( !$this->db->post_access_allow_check( $post_obj ) ){
		        $msg_type = get_post_meta( $post_obj->ID, 'postaccesscontroller_noacs_msg_type', true);
		        $post_obj->post_title   = 'Access Denied';
		        $post_obj->post_status  = 'private';
		        $post_obj->post_type    = 'noaccess';

		        $post_obj->post_content = $this->no_access_msg( $post->ID );
		        $post_obj->post_excerpt = $this->no_access_msg( $post->ID );

		        //COMMENTS handling code
					// Kill the comments template. This will deal with themes that don't check comment stati properly!
					add_filter( 'comments_template', postaccesscontroller_comments_template_override, 20 );
					// Remove comment-reply script for themes that include it indiscriminately
					wp_deregister_script( 'comment-reply' );

		    }

		    return $post_obj;

		}// check

		public function posts_join( $clause='' ) {
		    global $wpdb;

		    // echo '123';

		    // We join the postmeta table so we can check the value in the WHERE clause.
		    $clause .= " LEFT JOIN $wpdb->postmeta AS postaccesscontroller_ctrl_type ON ($wpdb->posts.ID = postaccesscontroller_ctrl_type.post_id AND postaccesscontroller_ctrl_type.meta_key = 'postaccesscontroller_ctrl_type') ";

		    return $clause;
		}

		public function posts_where( $clause = '' ) {

		    global $wpdb;

		    if( ( is_admin() && $this->check_user_role( 'administrator' ) ) || is_single() || is_page() ){

		    	$clause .= '';

		    }else{

			    // prep
			    $clause .= " AND ( ";

			    //then we'll do each "way" that users could have access to this and wrap each in their own parentheses

			    /*********************************************************
		            NO PAC SET
			     ********************************************************/
					$clause .= "( postaccesscontroller_ctrl_type.meta_value = 'none' )";

			    /*********************************************************
		            PUBLIC / NONE
			     ********************************************************/
					$clause .= " or ";
					$clause .= "( postaccesscontroller_ctrl_type.meta_value is null )";

			    /*********************************************************
		            USER
			     ********************************************************/
				    $clause .= " or ";

				    //put this in a subquery for clarity of what we're doing
				    //this gets all the meta_values (ie user_ids) from each post's metadata
				    $user_subquery = "select meta_value from $wpdb->postmeta where post_id = $wpdb->posts.ID and meta_key = 'postaccesscontroller_meta_user'";

				    //and then if the current post access control type is set to 'user' we check that query for the current user's ID
					$clause .= "( postaccesscontroller_ctrl_type.meta_value = 'user' and ".get_current_user_id()." in ($user_subquery) )";

			    /*********************************************************
		            GROUP
			     ********************************************************/
				    $clause .= " or ";

				    //build this piece by piece so it's more understandable what we're doing
				    //we'll start with getting just the query for all postmeta records for the current post that will tell us what "group" posts to get
				    $group_query = "select meta_value from $wpdb->postmeta where post_id = $wpdb->posts.ID and meta_key = 'postaccesscontroller_meta_group'";

				    //so then we need to get all the "user" meta values from the sum of all of those groups
				    $group_query = "select meta_value from $wpdb->postmeta where post_id in ($group_query) and meta_key = 'postaccesscontroller_group_user'";

				   	//and finally we can use that query to be the inner query here where we need to compare the current user ID to a list of "allowed" IDs
					$clause .= "( postaccesscontroller_ctrl_type.meta_value = 'group' and ".get_current_user_id()." in ($group_query))";

			    //close
			    $clause .= " ) ";

		    }

		    return $clause;

		}

		private function check_user_role( $role, $user_id = null ) {

		    if ( is_numeric( $user_id ) )
			$user = get_userdata( $user_id );
		    else
		        $user = wp_get_current_user();

		    if ( empty( $user ) )
			return false;

		    return in_array( $role, (array) $user->roles );
		}

	}//postaccesscontroller_checks	

/* End of file */
/* Location: ./post-access-controller/classes/checks.php */